@@ -67,7 +67,8 @@ class cuDFInterface(PandasInterface):
             data, _, _ = PandasInterface.init(eltype, data, kdims, vdims)
             data = cudf.from_pandas(data)
 
-        ncols = len(data.columns)
+        columns = list(data.columns)
+        ncols = len(columns)
         index_names = [data.index.name]
         if index_names == [None]:
             index_names = ['index']
@@ -80,21 +81,21 @@ class cuDFInterface(PandasInterface):
             ndim = None
         nvdim = vdim_param.bounds[1] if isinstance(vdim_param.bounds[1], int) else None
         if kdims and vdims is None:
-            vdims = [c for c in data.columns if c not in kdims]
+            vdims = [c for c in columns if c not in kdims]
         elif vdims and kdims is None:
-            kdims = [c for c in data.columns if c not in vdims][:ndim]
+            kdims = [c for c in columns if c not in vdims][:ndim]
         elif kdims is None:
-            kdims = list(data.columns[:ndim])
+            kdims = list(columns[:ndim])
             if vdims is None:
-                vdims = [d for d in data.columns[ndim:((ndim+nvdim) if nvdim else None)]
+                vdims = [d for d in columns[ndim:((ndim+nvdim) if nvdim else None)]
                          if d not in kdims]
         elif kdims == [] and vdims is None:
-            vdims = list(data.columns[:nvdim if nvdim else None])
+            vdims = list(columns[:nvdim if nvdim else None])
 
         # Handle reset of index if kdims reference index by name
         for kd in kdims:
             kd = dimension_name(kd)
-            if kd in data.columns:
+            if kd in columns:
                 continue
             if any(kd == ('index' if name is None else name)
                    for name in index_names):
@@ -106,13 +107,13 @@ class cuDFInterface(PandasInterface):
 
         if kdims:
             kdim = dimension_name(kdims[0])
-            if eltype._auto_indexable_1d and ncols == 1 and kdim not in data.columns:
+            if eltype._auto_indexable_1d and ncols == 1 and kdim not in columns:
                 data = data.copy()
                 data.insert(0, kdim, np.arange(len(data)))
 
         for d in kdims+vdims:
             d = dimension_name(d)
-            if len([c for c in data.columns if c == d]) > 1:
+            if len([c for c in columns if c == d]) > 1:
                 raise DataError('Dimensions may not reference duplicated DataFrame '
                                 'columns (found duplicate %r columns). If you want to plot '
                                 'a column against itself simply declare two dimensions '
@@ -122,7 +123,10 @@ class cuDFInterface(PandasInterface):
 
     @classmethod
     def range(cls, dataset, dimension):
-        column = dataset.data[dataset.get_dimension(dimension, strict=True).name]
+        dimension = dataset.get_dimension(dimension, strict=True)
+        column = dataset.data[dimension.name]
+        if dimension.nodata is not None:
+            column = cls.replace_value(column, dimension.nodata)
         if column.dtype.kind == 'O':
             return np.NaN, np.NaN
         else:
@@ -136,13 +140,15 @@ class cuDFInterface(PandasInterface):
         data = dataset.data[dim.name]
         if not expanded:
             data = data.unique()
-            return data.to_array() if compute else data
+            return data.values_host if compute else data.values
         elif keep_index:
             return data
         elif compute:
-            return data.to_array()
-        return data
-
+            return data.values_host
+        try:
+            return data.values
+        except Exception:
+            return data.values_host
 
     @classmethod
     def groupby(cls, dataset, dimensions, container_type, group_type, **kwargs):
@@ -162,7 +168,7 @@ class cuDFInterface(PandasInterface):
         group_kwargs['dataset'] = dataset.dataset
 
         # Find all the keys along supplied dimensions
-        keys = product(*(dataset.data[dimensions[0]].unique() for d in dimensions))
+        keys = product(*(dataset.data[dimensions[0]].unique().values_host for d in dimensions))
 
         # Iterate over the unique entries applying selection masks
         grouped_data = []
@@ -185,7 +191,7 @@ class cuDFInterface(PandasInterface):
     def select_mask(cls, dataset, selection):
         """
         Given a Dataset object and a dictionary with dimension keys and
-        selection keys (i.e tuple ranges, slices, sets, lists or literals)
+        selection keys (i.e. tuple ranges, slices, sets, lists, or literals)
         return a boolean mask over the rows in the Dataset object that
         have been selected.
         """
@@ -193,7 +199,7 @@ class cuDFInterface(PandasInterface):
         for dim, sel in selection.items():
             if isinstance(sel, tuple):
                 sel = slice(*sel)
-            arr = cls.values(dataset, dim, compute=False)
+            arr = cls.values(dataset, dim, keep_index=True)
             if util.isdatetime(arr) and util.pd:
                 try:
                     sel = util.parse_datetime_selection(sel)
@@ -281,8 +287,8 @@ class cuDFInterface(PandasInterface):
             if not hasattr(reindexed, agg):
                 raise ValueError('%s aggregation is not supported on cudf DataFrame.' % agg)
             agg = getattr(reindexed, agg)()
-            data = dict(((col, [v]) for col, v in zip(agg.index, agg.to_array())))
-            df = util.pd.DataFrame(data, columns=list(agg.index))
+            data = dict(((col, [v]) for col, v in zip(agg.index.values_host, agg.to_array())))
+            df = util.pd.DataFrame(data, columns=list(agg.index.values_host))
 
         dropped = []
         for vd in vdims:
