@@ -20,7 +20,7 @@ from ...operation import interpolate_curve
 from ...util.transform import dim
 from ..mixins import AreaMixin, BarsMixin, SpikesMixin
 from ..util import compute_sizes, get_min_distance
-from .element import ElementPlot, ColorbarPlot, LegendPlot
+from .element import ElementPlot, ColorbarPlot, LegendPlot, OverlayPlot
 from .selection import BokehOverlaySelectionDisplay
 from .styles import (
     expand_batched_style, base_properties, line_properties, fill_properties,
@@ -143,7 +143,9 @@ class PointPlot(LegendPlot, ColorbarPlot):
         # marker in certain cases
         has_angles = False
         for (key, el), zorder in zip(element.data.items(), zorders):
-            self.param.set_param(**self.lookup_options(el, 'plot').options)
+            el_opts = self.lookup_options(el, 'plot').options
+            self.param.set_param(**{k: v for k, v in el_opts.items()
+                                    if k not in OverlayPlot._propagate_options})
             style = self.lookup_options(element.last, 'style')
             style = style.max_cycles(len(self.ordering))[zorder]
             eldata, elmapping, style = self.get_data(el, ranges, style)
@@ -379,7 +381,9 @@ class CurvePlot(ElementPlot):
 
         zorders = self._updated_zorders(overlay)
         for (key, el), zorder in zip(overlay.data.items(), zorders):
-            self.param.set_param(**self.lookup_options(el, 'plot').options)
+            el_opts = self.lookup_options(el, 'plot').options
+            self.param.set_param(**{k: v for k, v in el_opts.items()
+                                    if k not in OverlayPlot._propagate_options})
             style = self.lookup_options(el, 'style')
             style = style.max_cycles(len(self.ordering))[zorder]
             eldata, elmapping, style = self.get_data(el, ranges, style)
@@ -459,8 +463,8 @@ class SideHistogramPlot(HistogramPlot):
         doc="A list of plugin tools to use on the plot.")
 
     _callback = """
-    color_mapper.low = cb_data['geometry']['{axis}0'];
-    color_mapper.high = cb_data['geometry']['{axis}1'];
+    color_mapper.low = cb_obj['geometry']['{axis}0'];
+    color_mapper.high = cb_obj['geometry']['{axis}1'];
     source.change.emit()
     main_source.change.emit()
     """
@@ -479,15 +483,20 @@ class SideHistogramPlot(HistogramPlot):
                       if d is not None]
         dimension = color_dims[0] if color_dims else None
         cmapper = self._get_colormapper(dimension, element, {}, {})
-        if cmapper and dimension in element.dimensions():
+        if cmapper:
+            cvals = None
             if isinstance(dimension, dim):
-                dim_name = dimension.dimension.name
-                data[dim_name] = [] if self.static_source else dimension.apply(element)
+                if dimension.applies(element):
+                    dim_name = dimension.dimension.name
+                    cvals = [] if self.static_source else dimension.apply(element)
             else:
-                dim_name = dimension.name
-                data[dim_name] = [] if self.static_source else element.dimension_values(dimension)
-            mapping['fill_color'] = {'field': dim_name,
-                                     'transform': cmapper}
+                if dimension in element.dimensions():
+                    dim_name = dimension.name
+                    cvals = [] if self.static_source else element.dimension_values(dimension)
+            if cvals is not None:
+                data[dim_name] = cvals
+                mapping['fill_color'] = {'field': dim_name,
+                                         'transform': cmapper}
         return (data, mapping, style)
 
 
@@ -506,14 +515,13 @@ class SideHistogramPlot(HistogramPlot):
                  if isinstance(t, BoxSelectTool)]
         if not tools or not sources:
             return
-        box_select, main_source = tools[0], sources[0]
+        main_source = sources[0]
         handles = {'color_mapper': self.handles['color_mapper'],
                    'source': self.handles['source'],
                    'cds': self.handles['source'],
                    'main_source': main_source}
-        axis = 'y' if self.invert_axes else 'x'
-        callback = self._callback.format(axis=axis)
-        box_select.js_on_event("selectiongeometry", CustomJS(args=handles, code=callback))
+        callback = self._callback.format(axis='y' if self.invert_axes else 'x')
+        self.state.js_on_event("selectiongeometry", CustomJS(args=handles, code=callback))
         return ret
 
 
